

CREATE OR REPLACE FUNCTION spCAdj_New_FixJnl (
    _CAdjNo     VARCHAR,
    _JnlNo      VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
    _CAdjDate       DATE;
    _Amount         NUMERIC(19,4);
    _cashid         VARCHAR;
    _retval         SPStatus%ROWTYPE;
    _spret          SPStatus%ROWTYPE;
    _CrtBy          VARCHAR(10);
    _CrtDate        DATE;
    _UpdBy          VARCHAR(10);
    _UpdDate        DATE;
    _Count          INT;
    _JnlNote        VARCHAR(255);
    _BranchID       VARCHAR(10);
    _ExcRate        NUMERIC(19,4);
    _JnlType        VARCHAR;
    _CAdjAccNo      VARCHAR;
    _LockDate       DATE;
    _BPID           VARCHAR;
    _InOut          VARCHAR;
    _ExtNote        VARCHAR;
    _CrcID          VARCHAR;
    _FisRate        NUMERIC(19,4);
    _CAdjType       VARCHAR;
    _AccNo          VARCHAR;
    _PerID          VARCHAR;
    _DeptID         VARCHAR;
    _PrjID          VARCHAR;
    _IsBeg          BOOLEAN;

    _CAdjTypeDesc   VARCHAR;
    _CAdjNote       VARCHAR;
    _isDraft        BOOLEAN;

    _CashCrcID      VARCHAR;
BEGIN

    _IsBeg:=FALSE;

    SELECT INTO 
        _CashID, _CAdjDate, _Amount, _CAdjNote, _CrtBy,_CrtDate,_UpdBy,_UpdDate,_ExcRate,_BranchID, _CAdjAccNo, _BPID, _CrcID, _FisRate, _CAdjType, _PerID, _DeptID, _PrjID, _CAdjNote, _isDraft
        cashid, CAdjDate, cadjAmt, CAdjNote,CrtBy, CrtDate, UpdBy, UpdDate,excrate,BranchID,CAdjAccNo, BPID, CrcID, FisRate, CAdjType, PerID, DeptID, PrjID, CAdjNote, isDraft
    FROM CAdj WHERE CAdjNo=_CAdjNo;

    IF (_isDraft IS NOT NULL AND _isDraft=TRUE) THEN
        _retval:=(0, 'OK');
        RETURN NEXT _retval;
        RETURN;
    END IF;

    SELECT INTO _CashCrcID crcid FROM cash WHERE cashid=_CashID;
    IF (_CrcID<>_CashCrcID) THEN
        _retval.ErrNo:=50002;
        _retval.ErrMsg='Mata Uang Kas tidak sama dengan Mata Uang transaksi ! (spCAdj_New)';
        RAISE EXCEPTION '%', _retval.ErrMsg;
    END IF;

    SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='CADJ';
    IF(_LockDate IS NOT NULL AND _LockDate>_CAdjDate) THEN
        _retval.ErrNo:=50002;
        _retval.ErrMsg='Penyesuaian kas melebihi batas tanggal kunci transaksi ! (spCAdj_New)';
        RAISE EXCEPTION '%', _retval.ErrMsg;
    END IF;

    IF(_Amount>0) THEN
        _InOut:='I';
    ELSIF(_Amount<0) THEN
        _InOut:='O';
    END IF;

    SELECT INTO _CAdjTypeDesc CAdjTypeDesc FROM CAdjType WHERE CAdjType=_CAdjType;
    _ExtNote=_CAdjTypeDesc|| ' ' ||  COALESCE(_CAdjNote, '');

/** no casha karena hanya fix jurnal */

/***********************  AKUNTANSI *****************************************/
    IF isAccInstalled() THEN

        IF(_CAdjType='BEG') THEN
            _JnlType:='BEG';
            IF (_CAdjNote IS NULL OR _CAdjNote='') THEN
                _JnlNote:='Saldo Awal Kas/Bank '||':'||_CAdjNo;
            ELSE
                _JnlNote:='Saldo Awal Kas/Bank '||':'||_CAdjNo||E' \nKeterangan : '|| _CAdjNote;
            END IF;
            _AccNo:=fAccDefault('BEGBAL');
            _IsBeg:=TRUE;
        ELSE
            _JnlType:='CADJ';
            _AccNo:=_CAdjAccNo;
            IF (_CAdjNote IS NULL OR _CAdjNote='') THEN
                _JnlNote:='Jurnal Penyesuaian Kas '||_CAdjNo;
            ELSE
                _JnlNote:='Jurnal Penyesuaian Kas '||_CAdjNo||E' \nKeterangan : '|| _CAdjNote;
            END IF;
        END IF;

        SELECT INTO _spret * FROM spJnl_Insert(
            _JnlNo, _JnlType, _CAdjDate, 'CADJ', _CAdjNo,
            _JnlNote, true, true, _Amount*_ExcRate, 
            _CrtBy,_CrtDate,_UpdBy,_UpdDate,
            _PerID, _BranchID, _DeptID, _PrjID, _CrcID, _ExcRate, _FisRate
        );
        IF _spret.ErrNo<>0 THEN
            _retval.ErrNo:=_spret.ErrNo;
            _retval.ErrMsg:=_spret.ErrMsg;
            RAISE EXCEPTION '%', _retval.ErrMsg;
        END IF;

        _Count:=1;
        IF (_Amount<0) THEN
            SELECT INTO _retval * FROM spJnlD_Insert(_JnlNo,_Count, _AccNo, 'D', ABS(_Amount*_ExcRate), _DeptID, _PrjID, _Amount);
            _Count:=_Count+1;
            SELECT INTO _retval * FROM spJnlD_Insert(_JnlNo,_Count, fAccCash(_CashID, _InOut,_IsBeg), 'K', ABS(_Amount*_ExcRate), _DeptID, _PrjID, _Amount);
        ELSE
            SELECT INTO _retval * FROM spJnlD_Insert(_JnlNo,_Count, fAccCash(_CashID, _InOut,_IsBeg), 'D', _Amount*_ExcRate, _DeptID, _PrjID, _Amount);
            _Count:=_Count+1;
            SELECT INTO _retval * FROM spJnlD_Insert(_JnlNo,_Count, _AccNo, 'K', _Amount*_ExcRate, _DeptID, _PrjID, _Amount);

        END IF;

        SELECT INTO _spret * FROM spJnl_New(_JnlNo);
        IF _spret.ErrNo<>0 THEN
            _retval.ErrNo:=_spret.ErrNo;
            _retval.ErrMsg:=_spret.ErrMsg;
            RAISE EXCEPTION '%', _retval.ErrMsg;
        END IF;

    END IF; -- isAccInstalled()

    _retval:=(0, 'OK');
    RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;

/** 

ambil cadj yang nominal jurnalnya tidak sama
sp diatas adalah modif spCAdj_New untuk hanya reproses jurnal dengan nomor jurnal yang sama

*/
CREATE OR REPLACE FUNCTION spFixCAdjJnl(

) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
    _retval     SPStatus%ROWTYPE;
    _spret      SPStatus%ROWTYPE;

    cData       REFCURSOR;

    _CAdjNo     cadj.cadjno%TYPE;
    _JnlNo      jnl.jnlno%TYPE;
BEGIN
    _retval:=(0, 'OK');

    OPEN cData FOR
        SELECT c.cadjno, j.jnlno, c.cadjamt*c.excrate, j.jnlamt 
        FROM cadj c 
        JOIN jnl j ON j.reftype='CADJ' AND j.refno=c.cadjno
        WHERE ABS(c.cadjamt*c.excrate)!=j.jnlamt;
    LOOP
        FETCH cData INTO _CAdjNo, _JnlNo;
        EXIT WHEN NOT FOUND;

        SELECT INTO _spret * FROM spJnl_Void(_JnlNo);
        IF (_spret.ErrNo<>0) THEN
            _retval.ErrNo:=_spret.ErrNo;
            _retval.ErrMsg:=_spret.ErrMsg;
            RAISE EXCEPTION '%', _retval.ErrMsg;
        END IF;

        SELECT INTO _spret * FROM spCAdj_New_FixJnl(_CAdjNo, _JnlNo);
        IF (_spret.ErrNo<>0) THEN
            _retval.ErrNo:=_spret.ErrNo;
            _retval.ErrMsg:=_spret.ErrMsg;
            RAISE EXCEPTION '%', _retval.ErrMsg;
        END IF;
    END LOOP;
    CLOSE cData;

    RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


SELECT * FROM spFixCAdjJnl();


